;;; entities.lisp --- Entity helper functions

;;; Code:

;;;
;; Get a list of all properties attached to ENTITY. Will return NIL if the
;; entity does not exist or if it has no properties.
;;
;; @param string ENTITY The name of the entity to fetch properties for.
;;
;; @return list List of CONS cells containing property names and values.
(defun entity:get-properties (entity)
  "Get a list of all properties attached to ENTITY.
Returns NIL if the entity does not exist or has no properties."
  (entity-get-properties entity))

;;;
;; Check if ENTITY has PROPERTY set.
;;
;; @param string ENTITY The name of the entity to check.
;; @param string PROPERTY The name of the property to search for.
;;
;; @return bool T if the property exists, NIL if not.
(defun entity:has-property? (entity property)
  "Check if ENTITY has PROPERTY set."
  (assoc (string-upcase property) (entity:get-properties entity)))

(defun entity:get-property (entity property-name)
  "Get the property for ENTITY with PROPERTY-NAME."
  (cdr-assoc (string-upcase property-name) (entity:get-properties entity)))

(defun entity:delete (entity-tag)
  "Delete the entity with ENTITY-TAG."
  (world:send-signal :DELETE-ENTITY-BY-TAG entity-tag))

(defun entity:destroy (entity-tag)
  "Delete the entity identified by ENTITY-TAG and send an event."
  (event:emit :EVENT-ENTITY-DESTROYED %self% nil)
  (world:send-signal :DELETE-ENTITY-BY-TAG entity-tag))

;; Create a new entity and return its ID.
(define (entity:spawn)
    (world:send-signal :SPAWN-ENTITY))

(defun entity:set-tag (entity-id tag)
  "Set the tag for ENTITY-ID to TAG."
  (world:send-signal :SET-ENTITY-TAG entity-id tag))

(defun entity:create-from-template (template tag)
  "Create a new entity from TEMPLATE, assign TAG and return."
  (let ((new-entity (world:send-signal :SPAWN-ENTITY template)))
    (entity:set-tag new-entity tag)
    new-entity))

;; (defun entity:create-from-template (template tag)
;;   "Create a new entity from TEMPLATE and assign TAG."
;;   (define entity (world:send-signal :SPAWN-ENTITY template))
;;   (entity:set-tag entity tag)
;;   entity)


;; Low-Level entity stuff

(defun entity:add-component (entity component)
  "Add COMPONENT to ENTITY."
  (world:send-signal :ADD-COMPONENT entity component))

(defun entity:remove-component (entity component)
  "Remove COMPONENT from ENTITY."
  (world:send-signal :REMOVE-COMPONENT entity component))

(defun entity:get-component-field (entity component field)
  "Get ENTITY's COMPONENT FIELD value."
  (world:send-signal :GET-COMPONENT-FIELD entity component field))

(defun entity:set-component-field (entity component field value)
  "Set ENTITY's COMPONENT FIELD to VALUE."
  (world:send-signal :SET-COMPONENT-FIELD entity component field value))

(defun entity:increase-component-field (entity component field value)
  "Increase ENTITY's COMPONENT FIELD by VALUE."
  (world:send-signal :INCREASE-COMPONENT-FIELD entity component field value))

(defun entity:decrease-component-field (entity component field value)
  "Decrease ENTITY's COMPONENT FIELD to VALUE."
  (world:send-signal :DECREASE-COMPONENT-FIELD entity component field value))


;; Weird stuff


;; [todo] - None of these work!

(defun get-property (name)
  "Get the current entity's property"
  (get-entity-property %self% name))

(defun has-property? (name)
  "Check if the current entity has a property"
  (entity-has-property? %self% name))
